# Copyright 2018 Smithsonian Astrophysical Observatory
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following
# disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
import numpy as np
import pytest

from astropy import units as u
from numpy.testing import assert_array_equal

from rama import read, is_template
from rama.models.source import SourcePosition
from rama.models.coordinates import SpaceFrame
from rama.models.photdmalt import PhotometryFilter

@pytest.fixture
def source_data(make_data_path):
    return read(make_data_path('photdm-source-test.vot.xml'))
    
def test_global_source_position(source_data, recwarn):
    positions = source_data.find_instances(SourcePosition)
    
    assert len(positions) == 1
    pos = positions[0]
    assert not is_template(pos)  # positions are direct
    assert pos.coord.frame.name == 'icrs'
    assert pos.coord.ra.degree == 123.45
    assert pos.coord.dec.degree == 67.89
    
def test_global_filter(source_data, recwarn):
    filters = source_data.find_instances(PhotometryFilter)
    
    assert len(filters) == 1
    f = filters[0]
    assert f.name == 'F606W'
    assert f.band_name == 'R'
    
    assert f.spectral_location.value == 606
    assert f.spectral_location.unit == 'nm'
    
    tp = f.transmission_point[0]
    assert tp.spectral.value == 5432.0
    assert tp.spectral.unit == 'nm'
    assert tp.spectral_error.value == 84.6
    assert tp.spectral_error.unit == 'nm'

def test_global_source(source_data, recwarn):
    filters = source_data.find_instances(PhotometryFilter)
    
    # TBD add some solid test data for sources with luminosities
    
#TBD Add some tests for template data.
